@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -68,15 +68,27 @@ class DataAdapter(object):
     def collection(self, name, *args, **kwargs):
         raise exceptions.NotImplementedError()
 
+    def collection_a(self, name, *args, **kwargs):
+        raise exceptions.NotImplementedError()
+
     def reset(self):
         raise exceptions.NotImplementedError()
 
+    def reset_a(self):
+        raise exceptions.NotImplementedError()
+
     def get_db(self):
         raise exceptions.NotImplementedError()
 
+    def get_db_a(self):
+        raise exceptions.NotImplementedError()
+
     def drop_db(self, *args, **kwargs):
         raise exceptions.NotImplementedError()
 
+    def drop_db_a(self, *args, **kwargs):
+        raise exceptions.NotImplementedError()
+
     def object_id(self, value = None):
         if not value: return self._id()
         if not len(value) == 24:
@@ -121,15 +133,29 @@ class MongoAdapter(DataAdapter):
         collection = db[name]
         return MongoCollection(self, name, collection)
 
+    def collection_a(self, name, *args, **kwargs):
+        db = self.get_db_a()
+        collection = db[name]
+        return MongoCollection(self, name, collection)
+
     def reset(self):
         return mongo.reset_connection()
 
+    def reset_a(self):
+        return mongo.reset_connection_a()
+
     def get_db(self):
         return mongo.get_db()
 
+    def get_db_a(self):
+        return mongo.get_db_a()
+
     def drop_db(self, *args, **kwargs):
         return mongo.drop_db()
 
+    def drop_db_a(self, *args, **kwargs):
+        return mongo.drop_db_a()
+
     def object_id(self, value = None):
         if not value: return self._id()
         return mongo.object_id(value)
@@ -396,8 +422,16 @@ class TinyCollection(Collection):
     def _to_update(self, modification, object = None):
         object = object or dict()
         increments = modification.get("$inc", {})
+        mins = modification.get("$min", {})
+        maxs = modification.get("$max", {})
         for name, increment in legacy.iteritems(increments):
             value = object.get(name, 0)
             value += increment
             object[name] = value
+        for name, target in legacy.iteritems(mins):
+            value = object.get(name, 0)
+            object[name] = min(value, target)
+        for name, target in legacy.iteritems(maxs):
+            value = object.get(name, 0)
+            object[name] = max(value, target)
         return object
