@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -99,12 +99,34 @@ try: import urlparse as _urlparse
 except ImportError: import urllib.parse; _urlparse = urllib.parse
 
 PYTHON_3 = sys.version_info[0] >= 3
-""" Global variable that defines if the current python
-interpreter is at least python 3 compliant, this is used
+""" Global variable that defines if the current Python
+interpreter is at least Python 3 compliant, this is used
 to take some of the conversion decision for runtime """
 
+PYTHON_35 = sys.version_info[0] >= 3 and sys.version_info[1] >= 5
+""" Global variable that defines if the current Python
+interpreter is at least Python 3.5 compliant """
+
+PYTHON_36 = sys.version_info[0] >= 3 and sys.version_info[1] >= 6
+""" Global variable that defines if the current Python
+interpreter is at least Python 3.6 compliant """
+
+PYTHON_39 = sys.version_info[0] >= 3 and sys.version_info[1] >= 9
+""" Global variable that defines if the current Python
+interpreter is at least Python 3.9 compliant """
+
+PYTHON_ASYNC = PYTHON_35
+""" Global variable that defines if the current Python
+interpreter support the async/await syntax responsible
+for the easy to use async methods """
+
+PYTHON_ASYNC_GEN = PYTHON_36
+""" Global variable that defines if the current Python
+interpreter support the async/await generator syntax
+responsible for the async generator methods """
+
 PYTHON_V = int("".join([str(v) for v in sys.version_info[:3]]))
-""" The python version integer describing the version of
+""" The Python version integer describing the version of
 a the interpreter as a set of three integer digits """
 
 if PYTHON_3: LONG = int
@@ -378,7 +400,7 @@ class Orderable(tuple):
     """
     Simple tuple type wrapper that provides a simple
     first element ordering, that is compatible with
-    both the python 2 and python 3+ infra-structures.
+    both the Python 2 and Python 3+ infra-structures.
     """
 
     def __cmp__(self, value):
