@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -187,6 +187,10 @@ class MemoryHandler(logging.Handler):
             messages_s = len(_messages_l)
             if messages_s > self.max_length: _messages_l.pop()
 
+    def clear(self):
+        self.messages = collections.deque()
+        self.messages_l = dict()
+
     def get_latest(self, count = None, level = None):
         count = count or 100
         is_level = level and not legacy.is_string(level)
@@ -197,6 +201,19 @@ class MemoryHandler(logging.Handler):
         slice = itertools.islice(messages, 0, count)
         return list(slice)
 
+    def flush_to_file(self, path, count = None, level = None, clear = True):
+        messages = self.get_latest(level = level, count = count or 65536)
+        if not messages: return
+        file = open(path, "wb")
+        try:
+            for message in messages:
+                if legacy.is_unicode(message):
+                    message = legacy.bytes(message, "utf-8", force = True)
+                file.write(message + "\n")
+        finally:
+            file.close()
+        if clear: self.clear()
+
 class BaseFormatter(logging.Formatter):
     """
     The base Appier logging formatted used to add some extra
