@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -424,6 +424,12 @@ class Request(object):
         if insensitive: name = name.title()
         return name in self.out_headers
 
+    def set_headers_b(self):
+        content_type = self.get_content_type() or "text/plain"
+        cache_control = self.get_cache_control()
+        self.set_header("Content-Type", content_type)
+        if cache_control: self.set_header("Cache-Control", cache_control)
+
     def get_address(self, resolve = True, cleanup = True):
         """
         Retrieves the client (network) address associated with the
@@ -801,6 +807,16 @@ class Request(object):
         if not milliseconds: return duration
         return duration * 1000.0
 
+    @property
+    def in_length(self):
+        data = self.get_data()
+        if not data: return 0
+        return len(data)
+
+    @property
+    def out_length(self, safe = True):
+        return self.result_l or 0
+
     @property
     def asynchronous(self):
         return True if self.get_header("X-Async") else False
