@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -407,13 +407,26 @@ def ensure_pip(name, package = None, delayed = False):
         install_pip_s(package, delayed = delayed)
 
 def install_pip(package, delayed = False, isolated = True, user = None):
-    import pip #@UnusedImport
-    try: import pip._internal
-    except ImportError: pip._internal = None
+    try:
+        import pip
+        pip_internal = pip
+    finally:
+        pass
+    try:
+        import pip._internal
+        pip_internal = pip._internal
+    except ImportError:
+        pass
+    try:
+        import pip._internal.main
+        pip_internal = pip._internal.main
+    except ImportError:
+        pass
     user = config.conf("PIP_USER", False, cast = bool)
     args = ["install", package]
-    if hasattr(pip._internal, "main"): pip_main = pip._internal.main
-    else: pip_main = pip.main #@UndefinedVariable
+    if hasattr(pip_internal, "main"): pip_main = pip_internal.main
+    elif hasattr(pip, "main"): pip_main = pip.main #@UndefinedVariable
+    else: raise exceptions.OperationalError(message = "pip not found")
     if user: args.insert(1, "--user")
     if delayed:
         process = multiprocessing.Process(
@@ -843,6 +856,7 @@ def camel_to_underscore(camel, separator = "_", lower = True):
     conversion of the provided camel cased one.
     """
 
+    if not camel: return camel
     value = FIRST_CAP_REGEX.sub(r"\1" + separator + r"\2", camel)
     value = ALL_CAP_REGEX.sub(r"\1" + separator + r"\2", value)
     if lower: value = value.lower()
@@ -871,10 +885,38 @@ def camel_to_readable(camel, lower = False, capitalize = False):
     used to display a value to an end user.
     """
 
+    if not camel: return camel
     underscore = camel_to_underscore(camel, lower = lower)
     return underscore_to_readable(underscore, capitalize = capitalize)
 
-def underscore_to_readable(underscore, capitalize = False):
+def underscore_to_camel(underscore, lower = False):
+    """
+    Converts the provided underscore cased based value into
+    a normalized camel cased string.
+
+    An optional lower parameter may be provided to obtain a
+    lower came case version of the string.
+
+    This is useful as most of the python string standards
+    are compliant with the underscore strategy.
+
+    :type underscore: String
+    :param underscore: The underscore cased string that is going to be
+    converted into an camel case based string.
+    :type lower: bool
+    :param lower: If the the first letter of the resulting camel
+    case string should be lower case (lower camel case).
+    :rtype: String
+    :return: The camel case based string resulting from the
+    conversion of the provided underscore cased one.
+    """
+
+    if not underscore: return underscore
+    camel = underscore_to_readable(underscore, capitalize = True, separator = "")
+    if not lower: return camel
+    return camel[0].lower() + camel[1:]
+
+def underscore_to_readable(underscore, capitalize = False, separator = " "):
     """
     Converts the given underscore oriented string value
     into a readable one meaning that the returned value
@@ -889,16 +931,20 @@ def underscore_to_readable(underscore, capitalize = False):
     :type capitalize: bool
     :param capitalize: If all of the words should be capitalized
     or if instead only the first one should.
+    :type separator: String
+    :param separator: The separator to be used to join the multiple
+    parts of the resulting readable tokens.
     :rtype: String
     :return: The final human readable string that may be
     used to display a value to an end user.
     """
 
+    if not underscore: return underscore
     parts = underscore.split("_")
     parts = [part for part in parts if part]
     if capitalize: parts = [part[0].upper() + part[1:] for part in parts]
     else: parts[0] = parts[0][0].upper() + parts[0][1:]
-    return " ".join(parts)
+    return separator.join(parts)
 
 def quote(value, *args, **kwargs):
     """
@@ -989,6 +1035,38 @@ def unescape(value, escape = "\\"):
             result.append(char)
     return "".join(result)
 
+def count_unescape(value, sub, escape = "\\"):
+    """
+    Runs the sub string count operation on an escaped string
+    so that it takes into account the escaped values avoiding
+    them for the count operation.
+
+    :type value: String
+    :param value: The base string value to have the number of
+    occurrences of a sub string counted.
+    :type sub: String
+    :param sub: The sub string to be evaluated for occurrences,
+    this value should be constrained to strings of single character.
+    :type escape: String
+    :param escape: The "special" escape character that will allow the
+    delimiter to be also present in the choices selection.
+    :rtype: int
+    :return: The final count of occurrences of the sub string
+    taking into account the proper escaping of the string.
+    """
+
+    count = 0
+    iterator = iter(value)
+    for char in iterator:
+        if char == escape:
+            try:
+                next(iterator)
+            except StopIteration:
+                pass
+        elif char == sub:
+            count += 1
+    return count
+
 def split_unescape(value, delimiter = " ", max = -1, escape = "\\", unescape = True):
     """
     Splits the provided string around the delimiter character that
@@ -2123,11 +2201,14 @@ class FileTuple(tuple):
     def tell(self):
         return self._position
 
-    def save(self, path):
+    def save(self, path, close = True):
         contents = self[2]
-        file = open(path, "wb")
-        try: file.write(contents)
-        finally: file.close()
+        if legacy.is_string(path): file = open(path, "wb")
+        else: file = path
+        try:
+            file.write(contents)
+        finally:
+            if close: file.close()
 
     @property
     def name(self):
